/*
 * Copyright (c) 2003 - 2007 Israfil Consulting Services Corporation
 * Copyright (c) 2003 - 2007 Christian Edward Gruber
 * All Rights Reserved
 * 
 * This software is licensed under the Berkeley Standard Distribution license,
 * (BSD license), as defined below:
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this 
 *    list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of Israfil Consulting Services nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 * 
 * $Id: Pump.java 362 2007-01-21 06:37:33Z cgruber $
 */
package net.israfil.foundation.concurrent;

import java.util.concurrent.TimeUnit;

import net.israfil.foundation.core.Copyright;

/**
 * A worker that repeats and draws items from a producer and feeds it to a consumer
 *
 * @author <a href="mailto:cgruber@israfil.net">Christian Edward Gruber</a>
 */
@Copyright(years={"2006","2007"},owner="Israfil Consulting Services Corporation",license="BSD")
public abstract class AbstractPump<T> extends RepeatingWorker {
	
	protected Long timeout = 0L;
	protected TimeUnit timeoutUnit = TimeUnit.MILLISECONDS;
	
	public AbstractPump() {}
	
	public AbstractPump(String name, long timeout, TimeUnit timeoutUnit) {
		super(name);
		this.timeout = timeout;
		this.timeoutUnit = timeoutUnit;
	}
	
	@Override
	protected void validate() {
		super.validate();
		if (this.timeout == null)
			throw new IllegalStateException("Timeoutunit cannot be null.");
		if (this.timeoutUnit == null)
			throw new IllegalStateException("Timeoutunit cannot be null.");
	}
	
	@Override
	public boolean continueProcessing() { return isRunning(); }

	@Override
	public long iterationDelay() { return 0; }

	public long getTimeout() { return timeout; }

	public void setTimeout(long timeout) { this.timeout = timeout; }

	public TimeUnit getTimeoutUnit() { return timeoutUnit; }

	public void setTimeoutUnit(TimeUnit timeoutUnit) {
		this.timeoutUnit = timeoutUnit;
	}

	
}
