/*
 * Copyright (c) 2006, 2007 Israfil Consulting Services Corporation
 * Copyright (c) 2006, 2007 Christian Edward Gruber
 * All Rights Reserved
 *
 * This software is licensed under the Berkeley Standard Distribution license,
 * (BSD license), as defined below:
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of Israfil Consulting Services nor the names of its contributors
 *    may be used to endorse or promote products derived from this software without
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * $Id: Sink.java 356 2007-01-21 04:17:47Z cgruber $
 */
package net.israfil.foundation.concurrent;

import java.util.concurrent.TimeUnit;

import net.israfil.foundation.core.Copyright;

/**
 * A sink which blocks 
 *
 * @author <a href="mailto:cgruber@israfil.net">Christian Edward Gruber </a>
 */
@Copyright(years={"2006","2007"},owner="Israfil Consulting Services Corporation",license="BSD")
public interface Consumer<T> {
	 
	/**
	 *  Consumes an item of type T.  If the consumer is not ready, then it will
	 *  block the call until interrupted.  If the object is closed, then this 
	 *  method will return null.  This method will never return null except 
	 *  when it is closed.
	 *  
	 *  @param item The item to be consumed.
	 */
	public void consume(T item) throws InterruptedException;

	/**
	 *  Consumes an item of type T.  If the consumer is not ready, then it will
	 *  block the call until the timeout has been reached or the method is
	 *  interrupted.If the object is closed, then this  method will return 
	 *  null.  This method will never return null except when it is closed.
	 *  
	 *  @param item The item to be consumed.
	 *  @param timeout The length of time consume should attempt to operate.
	 *  @param timeoutUnit The unit by which timeout should be interpreted.
	 */
	public void consume(T item, long timeout, TimeUnit timeoutUnit) throws InterruptedException;
	 
}
