/*
 * Copyright (c) 2003, 2004, 2005 Israfil Consulting Services Corporation
 * Copyright (c) 2003, 2004, 2005 Christian Edward Gruber
 * All Rights Reserved
 * 
 * This software is licensed under the Berkeley Standard Distribution license,
 * (BSD license), as defined below:
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this 
 *    list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of Israfil Consulting Services nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 * 
 * $Id: OneShotWorker.java 365 2007-01-22 04:36:55Z cgruber $
 */
package net.israfil.foundation.concurrent;

import java.util.logging.Logger;

import net.israfil.foundation.core.Copyright;

/**
 * A Worker sub-class that implements a one-shot lifecycle, performing any
 * initialization, pre-actions, the desired work, any post-work actions, and
 * any de-initialization.
 * 
 * @author <a href="mailto:cgruber@israfil.net">Christian Edward Gruber </a>
 */
@Copyright(years={"2006"},owner="Israfil Consulting Services Corporation",license="BSD")
public abstract class OneShotWorker extends Worker {
	
	private static Logger logger = Logger.getLogger(OneShotWorker.class.getName());
	
	public OneShotWorker() { }
	
	public OneShotWorker(String name) {
		super(name);
	}
		
	public void executeLifecycle() throws InterruptedException {
		logger.finer(getName() + " about to pre-process.");
		preProcess();
		logger.finer(getName() + " completed pre-processing.");
		logger.finer(getName() + " about to process.");
		process();
		logger.finer(getName() + " completed processing.");
		logger.finer(getName() + " about to post-process.");
		postProcess();
		logger.finer(getName() + " completed post-processing.");
	}
	
	public void preProcess() { }
		
	public void postProcess() { }
	
}
