/*
 * Copyright (c) 2006, 2007 Israfil Consulting Services Corporation
 * Copyright (c) 2006, 2007 Christian Edward Gruber
 * All Rights Reserved
 * 
 * This software is licensed under the Berkeley Standard Distribution license,
 * (BSD license), as defined below:
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this 
 *    list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of Israfil Consulting Services nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 * 
 * $Id: Source.java 129 2006-12-31 23:20:02Z cgruber $
 */
package net.israfil.foundation.concurrent;

import java.util.concurrent.TimeUnit;

import net.israfil.foundation.core.Copyright;

/**
 * An object that consumes an object and processes it, then produces the result
 * upon request.  If the processor has no item in processing, it may consume 
 * an item.  If the processor has an item it produces it when requested.  In 
 * the converse cases the behaviour of the object depends on whether the 
 * processor is in its on state or its off state. 
 *
 * @author <a href="mailto:cgruber@israfil.net">Christian Edward Gruber </a>
 */
@Copyright(years={"2006","2007"},owner="Israfil Consulting Services Corporation",license="BSD")
public class ProcessingPipe<T> extends AbstractPipe<T> {

	protected Processor<T> processor;
	protected Integer timeout;
	protected TimeUnit timeoutUnit;
	
	public ProcessingPipe() {}
	
	public Processor<T> getProcessor() { return processor; }
	public void setProcessor(Processor<T> processor) {
		this.processor = processor;
	}
	public Integer getTimeout() { return timeout; }
	public void setTimeout(Integer timeout) { this.timeout = timeout; }
	public TimeUnit getTimeoutUnit() { return timeoutUnit; }
	public void setTimeoutUnit(TimeUnit timeoutUnit) {
		this.timeoutUnit = timeoutUnit;
	}

	public void consume(T item, long timeout, TimeUnit unit) throws InterruptedException {
		validateTimeout(timeout,unit);
		try {
			if (timeout > 0) getQueue().offer(item,timeout,unit);
			else getQueue().put(item);
		} catch (InterruptedException e) {
			throw e;
		}
	}

	public T produce(long timeout, TimeUnit unit) throws InterruptedException {
		validateTimeout(timeout,unit);
		T processedItem = null;
		try {
			if (timeout > 0) 
				processedItem = this.processor.process(getQueue().poll(timeout,unit),getTimeout(),getTimeoutUnit());
			else 
				processedItem = this.processor.process(getQueue().poll(),getTimeout(),getTimeoutUnit());
		} catch (InterruptedException e) {
			throw e;
		}
		return processedItem;
	}
	
}
