/*
 * Copyright (c) 2006, 2007 Israfil Consulting Services Corporation
 * Copyright (c) 2006, 2007 Christian Edward Gruber
 * All Rights Reserved
 * 
 * This software is licensed under the Berkeley Standard Distribution license,
 * (BSD license), as defined below:
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this 
 *    list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of Israfil Consulting Services nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 * 
 * $Id: Source.java 355 2007-01-21 02:41:03Z cgruber $
 */
package net.israfil.foundation.concurrent;

import java.util.concurrent.TimeUnit;

import net.israfil.foundation.core.Copyright;

/**
 * An object that is a source (producer) of objects
 * 
 * @author <a href="mailto:cgruber@israfil.net">Christian Edward Gruber </a>
 */
@Copyright(years={"2006","2007"},owner="Israfil Consulting Services Corporation",license="BSD")
public interface Producer<T> {
	 
	/**
	 *  Produces an item of type T.  If the producer is not ready, then it will
	 *  block the call until the timeout is interrupted, assuming this gated
	 *  object is open.  If the object is closed, then this method will return
	 *  null.  This method will never return null except when it is closed.
	 */
	public T produce() throws InterruptedException;

	/**
	 *  Produces an item of type T.  If the producer is not ready, then it will
	 *  block the call until the timeout is reached or the method is 
	 *  interrupted. If the object is closed, then this method will return
	 *  null.  This method will never return null except when it is closed.
	 */
	public T produce(long timeout, TimeUnit unit) throws InterruptedException;

}
