/*
 * Copyright (c) 2006 Israfil Consulting Services Corporation
 * Copyright (c) 2006 Christian Edward Gruber
 * All Rights Reserved
 * 
 * This software is licensed under the Berkeley Standard Distribution license,
 * (BSD license), as defined below:
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice, this 
 *    list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 * 3. Neither the name of Israfil Consulting Services nor the names of its contributors 
 *    may be used to endorse or promote products derived from this software without 
 *    specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 * 
 * $Id: Dynamic.java 14 2006-01-27 23:50:37Z cgruber $
 */
package net.israfil.foundation.concurrent.producers;

import java.io.IOException;
import java.io.Reader;
import java.nio.CharBuffer;
import java.util.concurrent.TimeUnit;

import net.israfil.foundation.concurrent.Producer;

/**
 * A provider that reads an input stream and creates strings out of lines
 * of text.
 * 
 * @author <a href="cgruber@israfil.net">Christian Edward Gruber</a>
 */
public class ReaderStringProducer implements Producer<String> {
	Reader reader = null;
	public ReaderStringProducer(Reader reader) {
		this.reader = reader;
	}
	public String produce() throws InterruptedException {
		CharBuffer cb = CharBuffer.allocate(1024);
		int count = 0;
		try {
			int val = reader.read();
			while (val != -1) {
				cb.append( (char)val );
				count++;
				if ((char)val == '\n') break;
				val = reader.read();
			}
			if (count == 0 && val == -1) return null;
		} catch (IOException e) { return null; } 
		char[] finalarray = new char[count];
		cb.position(0);
		cb.get(finalarray);
		return new String(finalarray);
	}
	public String produce(long timeout, TimeUnit unit) throws InterruptedException {
		throw new UnsupportedOperationException("Timeouts not supported. Use produce()");
	}
	


 }
	